package meerkat;

import com.google.protobuf.Timestamp;
import meerkat.Registry.BooleanCallBack;
import meerkat.Registry.CollectionMessagesUtils;
import meerkat.Registry.RegistryTags;
import meerkat.Registry.RelevantDataCallBack;
import meerkat.bulletinboard.AsyncBulletinBoardClient;
import meerkat.crypto.DigitalSignature;
import meerkat.protobuf.BulletinBoardAPI.BulletinBoardMessage;
import meerkat.protobuf.BulletinBoardAPI.UnsignedBulletinBoardMessage;
import meerkat.protobuf.Crypto;
import meerkat.protobuf.VoterRegistry.*;

import java.io.InputStream;
import java.security.SignatureException;
import java.util.ArrayList;
import java.util.List;

/**
 * TODO : add ability to use DB of certificates
 */

/**
 * Created by Vladimir Eliezer Tokarev on 1/8/2016.
 * Gives the ability to synchronously manage voters information
 */
public class SimpleRegistry  implements VoterRegistry{

    protected DigitalSignature signer;
    protected AsyncBulletinBoardClient bulletinBoardClient ;
    protected Crypto.Signature  signature;
    protected InputStream certificateStream;

    public SimpleRegistry(DigitalSignature signer,
                          AsyncBulletinBoardClient communicator,
                          InputStream certificateStream) {
        this.signer = signer;
        this.bulletinBoardClient  = communicator;
        this.certificateStream = certificateStream;
    }

    /**
     * Creates BulletinBoardMessage with signed basicMessage and
     *         UnsignedBulletinBoardMessage that contains the basic message
     *
     * @param basicMessage BasicMessage
     * @return BulletinBoardMessage
     */
    protected BulletinBoardMessage CreateBulletinBoardMessage(UnsignedBulletinBoardMessage basicMessage) {
        try {
            BulletinBoardMessage.Builder bulletinBoardMessage = BulletinBoardMessage.newBuilder();

            signer.updateContent(basicMessage);
            signature = signer.sign();
            bulletinBoardMessage.setMsg(basicMessage).addSig(signature);

            return bulletinBoardMessage.build();
        } catch (SignatureException e) {
            e.printStackTrace();
            return null;
        }
    }

    public void AddVoter(VoterInfo voterInfo, RegistryCallBack callback) {
        UnsignedBulletinBoardMessage.Builder basicMessage =
                UnsignedBulletinBoardMessage.newBuilder().
                        addTag(RegistryTags.ID_TAG + voterInfo.getId().getId())
                        .addTag(RegistryTags.VOTER_ENTRY_TAG)
                        .addTag(RegistryTags.VOTER_DATA_TAG + voterInfo.getInfo())
                        .setTimestamp(Timestamp.newBuilder().setNanos((int) System.nanoTime()).build());

        bulletinBoardClient.postMessage(CreateBulletinBoardMessage(basicMessage.build()), new BooleanCallBack(callback));
    }

    public void AddToGroups(VoterRegistryMessage voterGroup, RegistryCallBack callback) {
        UnsignedBulletinBoardMessage.Builder basicMessage =
                UnsignedBulletinBoardMessage.newBuilder()
                        .addTag(RegistryTags.ID_TAG + voterGroup.getVoterID().getId())
                        .addTag(RegistryTags.ADD_TO_GROUP_TAG)
                        .setTimestamp(Timestamp.newBuilder().setNanos((int) System.nanoTime()).build());

        for ( int i = 0 ; i < voterGroup.getGroupIDCount() ; i++)
        {
            basicMessage.addTag(RegistryTags.GROUP_ID_TAG + voterGroup.getGroupID(i).getId());
        }

        bulletinBoardClient.postMessage(CreateBulletinBoardMessage(basicMessage.build()), new BooleanCallBack(callback));
    }

    public void SetVoted(VoterID voterId, RegistryCallBack callback) {
        UnsignedBulletinBoardMessage.Builder basicMessage =
                UnsignedBulletinBoardMessage.newBuilder()
                        .addTag(RegistryTags.ID_TAG + voterId.getId())
                        .addTag(RegistryTags.VOTE_ACTION_TAG)
                        .setTimestamp(Timestamp.newBuilder().setNanos((int) System.nanoTime()).build());

        bulletinBoardClient.postMessage(CreateBulletinBoardMessage(basicMessage.build()), new BooleanCallBack(callback));
    }

    public void GetGroups(GroupID groupID, RegistryCallBack callback) {
        List<String> GroupsActionsTags = new ArrayList<String>(2) {{
           add(RegistryTags.GROUP_ID_TAG + groupID.getId());
        }};
        bulletinBoardClient.readMessages(CollectionMessagesUtils.GenerateFiltersFromTags(GroupsActionsTags),
                new RelevantDataCallBack(callback, signer, signature, certificateStream));
    }

    public void GetPersonIDDetails(VoterID voterID, RegistryCallBack callback) {
        List<String> GroupsActionsTags = new ArrayList<String>() {{
            add(RegistryTags.ID_TAG + voterID.getId());
            add(RegistryTags.VOTER_ENTRY_TAG);
        }};
        bulletinBoardClient.readMessages(CollectionMessagesUtils.GenerateFiltersFromTags(GroupsActionsTags),
                new RelevantDataCallBack(callback, signer, signature, certificateStream));
    }
}
